
import React from 'react'
import { createRoot } from 'react-dom/client'
import App from './App.tsx'
import './index.css'

// Create a more comprehensive performance tracking system
performance.mark('app-init-start');

// Enhanced error boundary with more detailed error information
class ErrorBoundary extends React.Component<{children: React.ReactNode}, {hasError: boolean, error: Error | null, errorInfo: React.ErrorInfo | null}> {
  constructor(props: {children: React.ReactNode}) {
    super(props);
    this.state = { hasError: false, error: null, errorInfo: null };
  }

  static getDerivedStateFromError(error: Error) {
    // Update state so the next render will show the fallback UI
    return { hasError: true, error };
  }

  componentDidCatch(error: Error, errorInfo: React.ErrorInfo) {
    // Log the error to the console with more details
    console.error("React Error Boundary caught an error:", error);
    console.error("Component stack trace:", errorInfo.componentStack);
    
    this.setState({
      errorInfo: errorInfo
    });
  }

  render() {
    if (this.state.hasError) {
      // Enhanced fallback UI with more detailed error information
      return (
        <div className="min-h-screen flex flex-col items-center justify-center p-4 bg-gray-50 text-gray-800 dark:bg-gray-900 dark:text-gray-100">
          <div className="max-w-md w-full p-6 bg-white dark:bg-gray-800 rounded-lg shadow-lg">
            <h1 className="text-2xl font-bold mb-4 text-red-600 dark:text-red-400">Something went wrong</h1>
            <p className="mb-4">We're sorry, but there was an error loading the application.</p>
            <div className="bg-gray-100 dark:bg-gray-700 p-3 rounded mb-4 overflow-auto max-h-48 text-sm">
              <code className="whitespace-pre-wrap break-all">{this.state.error?.toString()}</code>
              {this.state.errorInfo && (
                <details className="mt-2">
                  <summary className="cursor-pointer text-blue-600 dark:text-blue-400">View component stack</summary>
                  <pre className="mt-2 text-xs overflow-auto">{this.state.errorInfo.componentStack}</pre>
                </details>
              )}
            </div>
            <button 
              onClick={() => window.location.reload()}
              className="w-full py-2 bg-blue-600 hover:bg-blue-700 text-white rounded transition-colors"
            >
              Refresh the page
            </button>
          </div>
        </div>
      );
    }

    return this.props.children;
  }
}

// Main app rendering - improved initialization to prevent duplicate roots
const initApp = () => {
  try {
    const container = document.getElementById("root");
    if (!container) {
      console.error("Root element not found");
      return;
    }
    
    // Clear any existing content to prevent React errors
    if (container.hasChildNodes()) {
      console.warn("Root container has existing content, clearing before render");
      container.innerHTML = '';
    }
    
    const root = createRoot(container);
    
    // Wrap App with ErrorBoundary to prevent white screens on errors
    root.render(
      <ErrorBoundary>
        <React.StrictMode>
          <App />
        </React.StrictMode>
      </ErrorBoundary>
    );
    
    // Mark render complete
    performance.mark('app-render-complete');
    performance.measure('app-total-init-time', 'app-init-start', 'app-render-complete');
    
  } catch (error) {
    console.error('Application failed to mount:', error);
    
    // Simple fallback error display
    const errorEl = document.createElement('div');
    errorEl.style.cssText = 'padding:20px;margin:30px auto;max-width:500px;text-align:center;';
    errorEl.innerHTML = `
      <h2>Something went wrong</h2>
      <p>The application failed to load. Please try refreshing the page.</p>
      <button onclick="window.location.reload()" style="padding:8px 16px;background:#3498db;color:white;border:none;border-radius:4px;cursor:pointer;margin-top:16px;">
        Refresh Page
      </button>
    `;
    document.body.appendChild(errorEl);
  }
};

// Use Performance Observer instead of directly modifying React
if ('PerformanceObserver' in window) {
  const observer = new PerformanceObserver((list) => {
    const entries = list.getEntries();
    if (entries.length > 0 && process.env.NODE_ENV === 'development') {
      console.log('Component Performance:', entries);
    }
  });
  observer.observe({ entryTypes: ['measure', 'mark'] });
}

// Preload critical resources
const preloadCriticalResources = () => {
  const resources = [
    // Critical CSS
    { href: '/index.css', as: 'style' },
    // Critical images
    { href: '/lovable-uploads/d471235a-dda8-405b-8243-facbf95b619d.png', as: 'image' },
    { href: '/lovable-uploads/89406fc0-09bd-4082-b0f2-5ea089954c64.png', as: 'image' }
  ];
  
  resources.forEach(resource => {
    const link = document.createElement('link');
    link.rel = 'preload';
    link.href = resource.href;
    link.as = resource.as;
    document.head.appendChild(link);
  });
};

// Initialize preloading and app rendering
if (document.readyState === 'loading') {
  document.addEventListener('DOMContentLoaded', () => {
    preloadCriticalResources();
    initApp();
  });
} else {
  preloadCriticalResources();
  initApp();
}
